%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% LPA UNWRAPPING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% INPUTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% true signal: y_true
%% true signals derivatives on x and y : y_x,y_y 
%% cos and sin of the observed wrapped phase: z_x1,z_x2
%% Wrapped phase: z_est
%% derivatives of the wrapped phase: z_x_est,z_y_est
%% window sizes: HH
%% indicator of the used initialization: true_initialization
%% mask matrix: mask
%% gamma-parameter of ICI rule: gamma

%%%% OUTPUTS %%%%%%%%%%%%%%%%%%%%%%%%%
%% absolute phase estimate :phi_estimate
%% absolute phase estiamte derivatives: phi_x_estimate,phi_y_estimate
%% adaptive window sizes: h_opt
%% estimates (approximate) with constant window sizes: PHI_estimate_h

% V. Katkovnik, Tampere University of Technology, Tampere, Finland,
% September 2007

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi_estimate, phi_x_estimate,phi_y_estimate,h_opt,PHI_estimate_h]=...
    LPA_ICI_Unwrap(y_true,y_x,y_y,z_x1,z_x2,z_est,z_x_est,z_y_est,HH,true_initialization,mask,gamma);



ddelta=1;
disp('------------------------------------')
disp('Start LPA filtering')
disp('------------------------------------')

[yN,xN]=size(y_true);
 
% arrays
phi_estimate=zeros(yN,xN);

phi_x_estimate=zeros(yN,xN);
phi_y_estimate=zeros(yN,xN);
PHI_estimate_h=zeros(yN,xN,length(HH));



if true_initialization==1
c0_in=y_true(1,1);
c1_in=y_x(1,1)/ddelta; 
c2_in=y_y(1,1)/ddelta;
else
c0_in=z_est(1,1);
c1_in=z_x_est(1,1)/ddelta; 
c2_in=z_y_est(1,1)/ddelta;
end

sigma11=function_stdEst2D_phase(z_est,1); % estimation of the standard 
% deviation of the noise in the absolute phase
% Loops over the data grid: 
 for s_y=1:yN
%   %  s_y;
% 
% %for s_x=h+1:xN-h-1
 for s_x=1:xN
     %%%% PLACE FOR FUSING WITH Zstep algorithm %%%%%%%%%%%%%%%%%%%%%%%%
     if (s_y==1)&(s_x>1) %% INITIALIZATION OF THE RECURSIVE ESTIMSATION FOR THE FIRST ROW on x 
     
   
            c0_in=phi_estimate(s_y,s_x-1); %Phase d_new=d1+wrap(d2-d1)
            c1_in=phi_x_estimate(s_y,s_x-1); %Derivative on x
            c2_in=phi_y_estimate(s_y,s_x-1); %Derivative on y
            
        end
%     
        if (s_y>1)&(s_x==1) %% INITIALIZATION OF THE RECURSIVE ESTIMSATION FOR THE FIRST ROW on x 
     
   
            c0_in=phi_estimate(s_y-1,s_x); %Phase d_new=d1+wrap(d2-d1)
            c1_in=phi_x_estimate(s_y-1,s_x); %Derivative on x
            c2_in=phi_y_estimate(s_y-1,s_x); %Derivative on y
            
        end
     
    
    %%%%%%%%%%%%%%%%%%%%%% novel things %%%%%%
   % INITIALIZATION OF THE RECURSIVE ESTIMSATION FOR ROWS on x starting from the second one 
    if (s_y>1)&(s_x>1)
    
            c0_in=(phi_estimate(s_y-1,s_x)+phi_estimate(s_y,s_x-1))/2; %Phase d_new=d1+wrap(d2-d1)
            c1_in=(phi_x_estimate(s_y-1,s_x)+phi_x_estimate(s_y,s_x-1))/2; %Derivative on x
            c2_in=(phi_y_estimate(s_y-1,s_x)+phi_y_estimate(s_y,s_x-1))/2; %Derivative on y
            C(1)=c0_in; C(2)=c1_in;  C(3)=c2_in;            
    end
    



C_initial=[c0_in,c1_in,c2_in]';

%C_initial=[z_est(s_y,s_x),z_x_est(s_y,s_x),z_y_est(s_y,s_x)];

% Adaptive window size point-wise estimation
[phi_estimate_h,C_opt,h_opt_IN]=PHASE_LA_ESTIMATION(C_initial,z_x1,z_x2,s_y,s_x,HH,yN,xN,gamma,sigma11,mask);

phi_estimate(s_y,s_x)=C_opt(1);  % Phase estimates 
phi_x_estimate(s_y,s_x)=C_opt(2); % Derivative on x estimates
phi_y_estimate(s_y,s_x)=C_opt(3); % Derivetive on y estimates
PHI_estimate_h(s_y,s_x,1:length(HH))=phi_estimate_h'; % Estimates for the fixed window sizes
%h_opt_IIN(s_y,s_x)=h_opt_IN;
h_opt(s_y,s_x)=HH(h_opt_IN);  % Adaptive window sizes 


end
end