% PHASE_LA_ESTIMATION.m

% Point-wise adaptive window size estimation for masked and full neighbourhood
% Inputs:
% C - Estimate 
% C_initial - Initial Estimate
% Cosine and Sine observations data, z_x1=cos(z_est); z_x2=sin(z_est);
% Data size yN,xN
% Estimation point coordinates s_y,s_x,h
% HH a set of the window size parameters
% std of the noise - sigma
% mask matrix of the size yN,xN
% Outputs:
% Estimates with window sizes corresponding to the elements of HH- YY_h
% Adaptive window size estimates for the phase and the derivatives are the
% elements of the vector C_opt
% Indexes (numbers of HH) for adaptive window sizes h_opt_IN
%


function [YY_h,C_opt,h_opt_IN]=PHASE_LA_ESTIMATION(C_initial,z_x1,z_x2,s_y,s_x,HH,yN,xN,gamma,sigma,mask)

C_h=zeros(3,length(HH));
C(1)=C_initial(1);C(2)=C_initial(2);C(3)=C_initial(3);
C=C';
  s_h=0;
  for h=HH % Loop for the window sizes
           
      s_h=s_h+1;
    
dx=max(1-s_x,-h):min(h,xN-s_x); %% Grids for the masks taken boundary conditions
dy=max(1-s_y,-h):min(h,yN-s_y); %% For Hessian matrix And gradient

ddx=s_x+max(1-s_x,-h):s_x+min(h,xN-s_x); %% Grids of the function values taken boundary conditions
ddy=s_y+max(1-s_y,-h):s_y+min(h,yN-s_y);
    ddelta=1;
    
    local_mask=mask(ddy,ddx);
% matrices for the Gauss-Newton algorithm
% mask is taken into consideration
[X,Y]=meshgrid(dx,dy);  X=X*ddelta.*local_mask; Y=Y*ddelta.*local_mask;  
       
ones_0=ones(size(X)).*local_mask;   
H11=sum(sum(ones_0));
H12=sum(sum(X));
H13=sum(sum(Y));
H22=sum(sum(X.*X));
H23=sum(sum(X.*Y));
H33=sum(sum(Y.*Y));
H_GN=[H11 H12 H13; H12 H22 H23; H13 H23 H33]; % "Hessian" matrix
H=H_GN+eye(3)*.001;
H_inv=H^(-1);
iteration_number=20;
  for s_C=1:iteration_number % Loop for the recursive estimation 
          
 ph_est=C(1)*ones_0+C(2)*X+C(3)*Y;     
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% calculation of variables %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cos1=cos(ph_est).*local_mask; sin2=sin(ph_est).*local_mask;

ee1=z_x1(ddy,ddx).*local_mask-cos1; ee2=z_x2(ddy,ddx).*local_mask-sin2; 

cos1_2=cos(ph_est).^2.*local_mask; sin2_2=sin(ph_est).^2.*local_mask;
%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% GRADIENT %%%%%%%%%%%%%%%%%%%%%%%%%%%%

DJ_C0=sum(sum(ee1.*sin2+ee2.*(-cos1)));
DJ_C1=sum(sum((ee1.*sin2+ee2.*(-cos1)).*X));
DJ_C2=sum(sum((ee1.*sin2+ee2.*(-cos1)).*Y));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% HESSIAN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



gradient=[DJ_C0 DJ_C1 DJ_C2]';
alpha=.75;

C1=C;

C=C+alpha*H_inv*gradient; % Recursive estimation
%C(2)=threshold(C(2),pi*100); C(3)=threshold(C(3),pi*100); %; GOOD

cond1=abs(C1(1)-C(1))<0.01;
cond2=abs(C1(2)-C(2))<0.005;
cond3=abs(C1(3)-C(3))<0.005;

if (cond1&cond2&cond3)==1 % Stoping rule
    s_C;
    break
end


   end % end for C
   std_C1(s_h)=sqrt(H_inv(1,1));
   C_h(:,s_h)=C;
   YY_h(s_h)=C(1);
  end  % end for HH
  
% ICI adaptive window size selection 
   [C_opt1, h_opt_IN, std_opt]=function_ICI_1D(C_h(1,:)',std_C1'*sigma,gamma);
   C_opt(1)=C_opt1; C_opt(2)=C_h(2,h_opt_IN);C_opt(3)=C_h(3,h_opt_IN);
   C_opt=C_opt';
   