function   disc = dbuild(dh,dv) 
%          disc = dbuilddh,dv); 
%   Encode vertical and horizontal discontinuities/quality maps
%
%   dv    - vertical discontinuity matrix 
%           dv(i,j) = 1 means a vertical discontinuity 
%                           between site (i,j) and site (i,j-1);
%                           disch(i,j) \in [0,1] 
%   dh    - horizontal discontinuity matrix 
%           dh(i,j) = 1 means a horizontal discontinuity 
%                           between site (i,j) and site (i-1,j)
%                           discv \in [0 1]
%   Author J.M. Bioucas Dias 2000
%   Topic - Interferometry
%
%================================================================
% NOTE:
%
%  Build discontinuity matrix.
%  dh and dv are compacted in a single float by
%
%  disc = 1000*(1-dh)+1000*1001*(1-dv);
%
%  To recover dh and dv (scaled by 1000) apply:
%  dh = rem(disc,1001); 
%  dv = floor(disc/1001);


SCALE = 1000;
[M N] = size(dh);
for i=1:M
   for j = 1:N
      disc(i,j)  = SCALE*(1-dh(i,j))+SCALE*(SCALE+1)*(1-dv(i,j));
   end
end
